@@ -1,7 +1,7 @@
 /*
  * mdadm - manage Linux "md" devices aka RAID arrays.
  *
- * Copyright (C) 2001-2002 Neil Brown <neilb@cse.unsw.edu.au>
+ * Copyright (C) 2001-2006 Neil Brown <neilb@suse.de>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -35,8 +35,8 @@
 #include	<values.h>
 #include	<syslog.h>
 
-static void alert(char *event, char *dev, char *disc, char *mailaddr, char *cmd,
-		  int dosyslog);
+static void alert(char *event, char *dev, char *disc, char *mailaddr, char *mailfrom,
+		  char *cmd, int dosyslog);
 
 static char *percentalerts[] = { 
 	"RebuildStarted",
@@ -113,6 +113,7 @@ int Monitor(mddev_dev_t devlist,
 	} *statelist = NULL;
 	int finished = 0;
 	struct mdstat_ent *mdstat = NULL;
+	char *mailfrom = NULL;
 
 	if (!mailaddr) {
 		mailaddr = conf_get_mailaddr(config);
@@ -120,6 +121,8 @@ int Monitor(mddev_dev_t devlist,
 			fprintf(stderr, Name ": Monitor using email address \"%s\" from config file\n",
 			       mailaddr);
 	}
+	mailfrom = conf_get_mailfrom(config);
+
 	if (!alert_cmd) {
 		alert_cmd = conf_get_program(config);
 		if (alert_cmd && ! scan)
@@ -219,12 +222,12 @@ int Monitor(mddev_dev_t devlist,
 			unsigned int i;
 
 			if (test)
-				alert("TestMessage", dev, NULL, mailaddr, alert_cmd, dosyslog);
+				alert("TestMessage", dev, NULL, mailaddr, mailfrom, alert_cmd, dosyslog);
 			fd = open(dev, O_RDONLY);
 			if (fd < 0) {
 				if (!st->err)
 					alert("DeviceDisappeared", dev, NULL,
-					      mailaddr, alert_cmd, dosyslog);
+					      mailaddr, mailfrom, alert_cmd, dosyslog);
 /*					fprintf(stderr, Name ": cannot open %s: %s\n",
 						dev, strerror(errno));
 */				st->err=1;
@@ -233,7 +236,7 @@ int Monitor(mddev_dev_t devlist,
 			if (ioctl(fd, GET_ARRAY_INFO, &array)<0) {
 				if (!st->err)
 					alert("DeviceDisappeared", dev, NULL,
-					      mailaddr, alert_cmd, dosyslog);
+					      mailaddr, mailfrom, alert_cmd, dosyslog);
 /*					fprintf(stderr, Name ": cannot get array info for %s: %s\n",
 						dev, strerror(errno));
 */				st->err=1;
@@ -244,7 +247,7 @@ int Monitor(mddev_dev_t devlist,
 				array.level != 6 && array.level != 10) {
 				if (!st->err)
 					alert("DeviceDisappeared", dev, "Wrong-Level",
-					      mailaddr, alert_cmd, dosyslog);
+					      mailaddr, mailfrom, alert_cmd, dosyslog);
 				st->err = 1;
 				close(fd);
 				continue;
@@ -281,27 +284,27 @@ int Monitor(mddev_dev_t devlist,
 			    mse &&	/* is in /proc/mdstat */
 			    mse->pattern && strchr(mse->pattern, '_') /* degraded */
 				)
-				alert("DegradedArray", dev, NULL, mailaddr, alert_cmd, dosyslog);
+				alert("DegradedArray", dev, NULL, mailaddr, mailfrom, alert_cmd, dosyslog);
 
 			if (st->utime == 0 && /* new array */
 			    st->expected_spares > 0 && 
 			    array.spare_disks < st->expected_spares) 
-				alert("SparesMissing", dev, NULL, mailaddr, alert_cmd, dosyslog);
+				alert("SparesMissing", dev, NULL, mailaddr, mailfrom, alert_cmd, dosyslog);
 			if (mse &&
 			    st->percent == -1 && 
 			    mse->percent >= 0)
-				alert("RebuildStarted", dev, NULL, mailaddr, alert_cmd, dosyslog);
+				alert("RebuildStarted", dev, NULL, mailaddr, mailfrom, alert_cmd, dosyslog);
 			if (mse &&
 			    st->percent >= 0 &&
 			    mse->percent >= 0 &&
 			    (mse->percent / 20) > (st->percent / 20))
 				alert(percentalerts[mse->percent/20],
-				      dev, NULL, mailaddr, alert_cmd, dosyslog);
+				      dev, NULL, mailaddr, mailfrom, alert_cmd, dosyslog);
 
 			if (mse &&
 			    mse->percent == -1 &&
 			    st->percent >= 0)
-				alert("RebuildFinished", dev, NULL, mailaddr, alert_cmd, dosyslog);
+				alert("RebuildFinished", dev, NULL, mailaddr, mailfrom, alert_cmd, dosyslog);
 
 			if (mse)
 				st->percent = mse->percent;
@@ -335,19 +338,19 @@ int Monitor(mddev_dev_t devlist,
 					     ((st->devstate[i]&change)&(1<<MD_DISK_ACTIVE)) ||
 					     ((st->devstate[i]&change)&(1<<MD_DISK_SYNC)))
 						)
-						alert("Fail", dev, dv, mailaddr, alert_cmd, dosyslog);
+						alert("Fail", dev, dv, mailaddr, mailfrom, alert_cmd, dosyslog);
 					else if (i >= (unsigned)array.raid_disks &&
 						 (disc.major || disc.minor) &&
 						 st->devid[i] == makedev(disc.major, disc.minor) &&
 						 ((newstate&change)&(1<<MD_DISK_FAULTY))
 						)
-						alert("FailSpare", dev, dv, mailaddr, alert_cmd, dosyslog);
+						alert("FailSpare", dev, dv, mailaddr, mailfrom, alert_cmd, dosyslog);
 					else if (i < (unsigned)array.raid_disks &&
 						 (((st->devstate[i]&change)&(1<<MD_DISK_FAULTY)) ||
 						  ((newstate&change)&(1<<MD_DISK_ACTIVE)) ||
 						  ((newstate&change)&(1<<MD_DISK_SYNC)))
 						)
-						alert("SpareActive", dev, dv, mailaddr, alert_cmd, dosyslog);
+						alert("SpareActive", dev, dv, mailaddr, mailfrom, alert_cmd, dosyslog);
 				}
 				st->devstate[i] = disc.state;
 				st->devid[i] = makedev(disc.major, disc.minor);
@@ -382,6 +385,7 @@ int Monitor(mddev_dev_t devlist,
 						if (fd >=0) close(fd);
 						free(st->devname);
 						free(st);
+						put_md_name(st->devname);
 						continue;
 					}
 					close(fd);
@@ -393,7 +397,7 @@ int Monitor(mddev_dev_t devlist,
 					st->spare_group = NULL;
 					st->expected_spares = -1;
 					statelist = st;
-					alert("NewArray", st->devname, NULL, mailaddr, alert_cmd, dosyslog);
+					alert("NewArray", st->devname, NULL, mailaddr, mailfrom, alert_cmd, dosyslog);
 					new_found = 1;
 				}
 		}
@@ -434,7 +438,7 @@ int Monitor(mddev_dev_t devlist,
 								  (unsigned long)dev) == 0) {
 								if (ioctl(fd1, HOT_ADD_DISK,
 									  (unsigned long)dev) == 0) {
-									alert("MoveSpare", st->devname, st2->devname, mailaddr, alert_cmd, dosyslog);
+									alert("MoveSpare", st->devname, st2->devname, mailaddr, mailfrom, alert_cmd, dosyslog);
 									close(fd1);
 									close(fd2);
 									break;
@@ -460,7 +464,7 @@ int Monitor(mddev_dev_t devlist,
 }
 
 
-static void alert(char *event, char *dev, char *disc, char *mailaddr, char *cmd,
+static void alert(char *event, char *dev, char *disc, char *mailaddr, char *mailfrom, char *cmd,
 		  int dosyslog)
 {
 	int priority;
@@ -486,13 +490,18 @@ static void alert(char *event, char *dev, char *disc, char *mailaddr, char *cmd,
 	if (mailaddr && 
 	    (strncmp(event, "Fail", 4)==0 || 
 	     strncmp(event, "Test", 4)==0 ||
+	     strncmp(event, "Spares", 6)==0 ||
 	     strncmp(event, "Degrade", 7)==0)) {
 		FILE *mp = popen(Sendmail, "w");
 		if (mp) {
+			FILE *mdstat;
 			char hname[256];
 			gethostname(hname, sizeof(hname));
 			signal(SIGPIPE, SIG_IGN);
-			fprintf(mp, "From: " Name " monitoring <root>\n");
+			if (mailfrom)
+				fprintf(mp, "From: %s\n", mailfrom);
+			else
+				fprintf(mp, "From: " Name " monitoring <root>\n");
 			fprintf(mp, "To: %s\n", mailaddr);
 			fprintf(mp, "Subject: %s event on %s:%s\n\n", event, dev, hname);
 
@@ -505,6 +514,16 @@ static void alert(char *event, char *dev, char *disc, char *mailaddr, char *cmd,
 				fprintf(mp, "It could be related to component device %s.\n\n", disc);
 
 			fprintf(mp, "Faithfully yours, etc.\n");
+
+			mdstat = fopen("/proc/mdstat", "r");
+			if (mdstat) {
+				char buf[8192];
+				int n;
+				fprintf(mp, "\nP.S. The /proc/mdstat file current contains the following:\n\n");
+				while ( (n=fread(buf, 1, sizeof(buf), mdstat)) > 0)
+					fwrite(buf, 1, n, mp);
+				fclose(mdstat);
+			}
 			fclose(mp);
 		}
 
@@ -521,7 +540,8 @@ static void alert(char *event, char *dev, char *disc, char *mailaddr, char *cmd,
 			priority = LOG_CRIT;
 		/* Good to know about, but are not failures: */
 		else if (strncmp(event, "Rebuild", 7)==0 ||
-			 strncmp(event, "MoveSpare", 9)==0)
+			 strncmp(event, "MoveSpare", 9)==0 ||
+			 strncmp(event, "Spares", 6) != 0)
 			priority = LOG_WARNING;
 		/* Everything else: */
 		else
