@@ -1,7 +1,7 @@
 /*
  * mdadm - manage Linux "md" devices aka RAID arrays.
  *
- * Copyright (C) 2001-2002 Neil Brown <neilb@cse.unsw.edu.au>
+ * Copyright (C) 2001-2006 Neil Brown <neilb@suse.de>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -27,7 +27,7 @@
  *           Australia
  */
 
-#define	__USE_LARGEFILE64
+#define	_GNU_SOURCE
 #include	<unistd.h>
 #ifndef __dietlibc__
 extern __off64_t lseek64 __P ((int __fd, __off64_t __offset, int __whence));
@@ -99,6 +99,13 @@ struct mdinfo {
 	int			new_level, delta_disks, new_layout, new_chunk;
 };
 
+struct createinfo {
+	int	uid;
+	int	gid;
+	int	autof;
+	int	mode;
+};
+
 #define Name "mdadm"
 
 enum mode {
@@ -118,6 +125,21 @@ extern char Version[], Usage[], Help[], OptionHelp[],
 	Help_create[], Help_build[], Help_assemble[], Help_grow[],
 	Help_manage[], Help_misc[], Help_monitor[], Help_config[];
 
+/* for option that don't have short equivilents, we assign arbitrary
+ * small numbers.  '1' means an undecorated option, so we start at '2'.
+ */
+enum special_options {
+	AssumeClean = 2,
+	BitmapChunk,
+	WriteBehind,
+	ReAdd,
+	NoDegraded,
+	Sparc22,
+	BackupFile,
+	HomeHost,
+	AutoHomeHost,
+};
+
 /* structures read from config file */
 /* List of mddevice names and identifiers
  * Identifiers can be:
@@ -161,6 +183,7 @@ typedef struct mddev_dev_s {
 				 */
 	char writemostly;
 	char re_add;
+	char used;		/* set when used */
 	struct mddev_dev_s *next;
 } *mddev_dev_t;
 
@@ -251,15 +274,18 @@ extern char *map_dev(int major, int minor, int create);
 
 
 extern struct superswitch {
-	void (*examine_super)(void *sbv);
+	void (*examine_super)(void *sbv, char *homehost);
 	void (*brief_examine_super)(void *sbv);
-	void (*detail_super)(void *sbv);
+	void (*detail_super)(void *sbv, char *homehost);
 	void (*brief_detail_super)(void *sbv);
 	void (*uuid_from_super)(int uuid[4], void *sbv);
 	void (*getinfo_super)(struct mdinfo *info, void *sbv);
-	int (*update_super)(struct mdinfo *info, void *sbv, char *update, char *devname, int verbose);
+	int (*match_home)(void *sbv, char *homehost);
+	int (*update_super)(struct mdinfo *info, void *sbv, char *update,
+			    char *devname, int verbose,
+			    int uuid_set, char *homehost);
 	__u64 (*event_super)(void *sbv);
-	int (*init_super)(struct supertype *st, void **sbp, mdu_array_info_t *info, unsigned long long size, char *name);
+	int (*init_super)(struct supertype *st, void **sbp, mdu_array_info_t *info, unsigned long long size, char *name, char *homehost);
 	void (*add_to_super)(void *sbv, mdu_disk_info_t *dinfo);
 	int (*store_super)(struct supertype *st, int fd, void *sbv);
 	int (*write_init_super)(struct supertype *st, void *sbv, mdu_disk_info_t *dinfo, char *devname);
@@ -292,7 +318,6 @@ struct stat64;
   struct FTW {};
 # define FTW_PHYS 1
 #else
-# define  __USE_XOPEN_EXTENDED
 # include <ftw.h>
 # ifdef __dietlibc__
 #  define FTW_PHYS 1
@@ -322,7 +347,7 @@ extern int Assemble(struct supertype *st, char *mddev, int mdfd,
 		    char *conffile,
 		    mddev_dev_t devlist, char *backup_file,
 		    int readonly, int runstop,
-		    char *update,
+		    char *update, char *homehost,
 		    int verbose, int force);
 
 extern int Build(char *mddev, int mdfd, int chunk, int level, int layout,
@@ -333,15 +358,15 @@ extern int Build(char *mddev, int mdfd, int chunk, int level, int layout,
 
 extern int Create(struct supertype *st, char *mddev, int mdfd,
 		  int chunk, int level, int layout, unsigned long long size, int raiddisks, int sparedisks,
-		  char *name,
+		  char *name, char *homehost,
 		  int subdevs, mddev_dev_t devlist,
 		  int runstop, int verbose, int force, int assume_clean,
 		  char *bitmap_file, int bitmap_chunk, int write_behind, int delay);
 
-extern int Detail(char *dev, int brief, int test);
+extern int Detail(char *dev, int brief, int test, char *homehost);
 extern int Query(char *dev);
 extern int Examine(mddev_dev_t devlist, int brief, int scan, int SparcAdjust,
-		   struct supertype *forcest);
+		   struct supertype *forcest, char *homehost);
 extern int Monitor(mddev_dev_t devlist,
 		   char *mailaddr, char *alert_cmd,
 		   int period, int daemonise, int scan, int oneshot,
@@ -355,6 +380,7 @@ extern int CreateBitmap(char *filename, int force, char uuid[16],
 			unsigned long long array_size,
 			int major);
 extern int ExamineBitmap(char *filename, int brief, struct supertype *st);
+extern void bitmap_update_uuid(int fd, int *uuid);
 
 extern int md_get_version(int fd);
 extern int get_linux_version(void);
@@ -367,11 +393,14 @@ extern int get_mdp_major(void);
 extern int dev_open(char *dev, int flags);
 extern int is_standard(char *dev, int *nump);
 
-
+extern int parse_auto(char *str, char *msg, int config);
 extern mddev_ident_t conf_get_ident(char *conffile, char *dev);
 extern mddev_dev_t conf_get_devs(char *conffile);
+extern struct createinfo *conf_get_create_info(char *conffile);
 extern char *conf_get_mailaddr(char *conffile);
+extern char *conf_get_mailfrom(char *conffile);
 extern char *conf_get_program(char *conffile);
+extern char *conf_get_homehost(char *conffile);
 extern char *conf_line(FILE *file);
 extern char *conf_word(FILE *file, int allow_key);
 extern void free_line(char *line);
